@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2001,2002,2003 Nikos Mavroyanopoulos
+ * Copyright (C) 2004 Free Software Foundation
  *
  * This file is part of GNUTLS.
  *
@@ -57,22 +58,23 @@ gnutls_extension_entry _gnutls_extensions[MAX_EXT_SIZE] = {
 
 /* EXTENSION functions */
 
-void* _gnutls_ext_func_recv(uint16 type)
+ext_recv_func _gnutls_ext_func_recv(uint16 type)
 {
-	void* ret=NULL;
+	ext_recv_func ret = NULL;
 	GNUTLS_EXTENSION_LOOP(ret = p->gnutls_ext_func_recv);
 	return ret;
 
 }
-void* _gnutls_ext_func_send(uint16 type)
+
+ext_send_func _gnutls_ext_func_send(uint16 type)
 {
-	void* ret=NULL;
+	ext_send_func ret = NULL;
 	GNUTLS_EXTENSION_LOOP(ret = p->gnutls_ext_func_send);
 	return ret;
 
 }
 
-const char *_gnutls_extension_get_name(uint16 type)
+const char* _gnutls_extension_get_name(uint16 type)
 {
 	const char *ret = NULL;
 
@@ -103,7 +105,7 @@ int next, ret;
 int pos=0;
 uint16 type;
 const opaque* sdata;
-int (*ext_func_recv)( gnutls_session, const opaque*, int);
+ext_recv_func ext_recv;
 uint16 size;
 
 #ifdef DEBUG
@@ -143,9 +145,9 @@ int i;
 		sdata = &data[pos];
 		pos+=size;
 		
-		ext_func_recv = _gnutls_ext_func_recv(type);
-		if (ext_func_recv == NULL) continue;
-		if ( (ret=ext_func_recv( session, sdata, size)) < 0) {
+		ext_recv = _gnutls_ext_func_recv(type);
+		if (ext_recv == NULL) continue;
+		if ( (ret=ext_recv( session, sdata, size)) < 0) {
 			gnutls_assert();
 			return ret;
 		}
@@ -172,43 +174,52 @@ static void _gnutls_extension_list_add( gnutls_session session, uint16 type) {
 	}
 }
 
-int _gnutls_gen_extensions( gnutls_session session, opaque** data) {
+int _gnutls_gen_extensions( gnutls_session session, opaque* data, size_t data_size) 
+{
 int next, size;
 uint16 pos=0;
-opaque sdata[1024];
-int sdata_size = sizeof(sdata);
-int (*ext_func_send)( gnutls_session, opaque*, int);
+opaque *sdata;
+int sdata_size;
+ext_send_func ext_send;
 
 
-	(*data) = gnutls_malloc(2); /* allocate size for size */
-	if ((*data)==NULL) {
+	if (data_size < 2) {
+		gnutls_assert();
+		return GNUTLS_E_INTERNAL_ERROR;
+	}
+
+	/* allocate enough data for each extension.
+	 */
+	sdata_size = data_size;
+	sdata = gnutls_malloc( sdata_size);
+	if (sdata == NULL) {
 		gnutls_assert();
 		return GNUTLS_E_MEMORY_ERROR;
 	}
-
+	
 	pos+=2;
 	next = MAX_EXT_TYPES; /* maximum supported extensions */
 	do {
 		next--;
-		ext_func_send = _gnutls_ext_func_send(next);
-		if (ext_func_send == NULL) continue;
-		size = ext_func_send( session, sdata, sdata_size);
+		ext_send = _gnutls_ext_func_send(next);
+		if (ext_send == NULL) continue;
+		size = ext_send( session, sdata, sdata_size);
 		if (size > 0) {
-			(*data) = gnutls_realloc_fast( (*data), pos+size+4);
-			if ((*data)==NULL) {
+			if (data_size < pos+(size_t)size+4) {
 				gnutls_assert();
-				return GNUTLS_E_MEMORY_ERROR;
+				gnutls_free(sdata);
+				return GNUTLS_E_INTERNAL_ERROR;
 			}
 
 			/* write extension type */
-			_gnutls_write_uint16( next, &(*data)[pos]);
+			_gnutls_write_uint16( next, &data[pos]);
 			pos+=2;
 			
 			/* write size */
-			_gnutls_write_uint16( size, &(*data)[pos]);
+			_gnutls_write_uint16( size, &data[pos]);
 			pos+=2;
 			
-			memcpy( &(*data)[pos], sdata, size);
+			memcpy( &data[pos], sdata, size);
 			pos+=size;
 			
 			/* add this extension to the extension list
@@ -219,7 +230,7 @@ int (*ext_func_send)( gnutls_session, opaque*, int);
 				_gnutls_extension_get_name(next));
 		} else if (size < 0) {
 			gnutls_assert();
-			gnutls_free(*data); *data = NULL;
+			gnutls_free(sdata);
 			return size;
 		}
 		
@@ -228,13 +239,13 @@ int (*ext_func_send)( gnutls_session, opaque*, int);
 	size = pos;
 	pos-=2; /* remove the size of the size header! */
 
-	_gnutls_write_uint16( pos, (*data));
+	_gnutls_write_uint16( pos, data);
 
 	if (size==2) { /* empty */
 		size = 0;
-		gnutls_free(*data);
-		(*data) = NULL;
 	}
+	
+	gnutls_free( sdata);
 	return size;
 
 }
